@@ -62,9 +62,8 @@ static ignorelist_t *il_device = NULL;
 static ignorelist_t *il_mountpoint = NULL;
 static ignorelist_t *il_fstype = NULL;
 
-static _Bool by_device = false;
-static _Bool report_reserved = false;
-static _Bool report_inodes = false;
+static _Bool by_device = 0;
+static _Bool report_inodes = 0;
 
 static int df_init (void)
 {
@@ -119,25 +118,16 @@ static int df_config (const char *key, const char *value)
 	else if (strcasecmp (key, "ReportByDevice") == 0)
 	{
 		if (IS_TRUE (value))
-			by_device = true;
-
-		return (0);
-	}
-	else if (strcasecmp (key, "ReportReserved") == 0)
-	{
-		if (IS_TRUE (value))
-			report_reserved = true;
-		else
-			report_reserved = false;
+			by_device = 1;
 
 		return (0);
 	}
 	else if (strcasecmp (key, "ReportInodes") == 0)
 	{
 		if (IS_TRUE (value))
-			report_inodes = true;
+			report_inodes = 1;
 		else
-			report_inodes = false;
+			report_inodes = 0;
 
 		return (0);
 	}
@@ -146,28 +136,6 @@ static int df_config (const char *key, const char *value)
 	return (-1);
 }
 
-static void df_submit_two (char *df_name,
-		const char *type,
-		gauge_t df_used,
-		gauge_t df_free)
-{
-	value_t values[2];
-	value_list_t vl = VALUE_LIST_INIT;
-
-	values[0].gauge = df_used;
-	values[1].gauge = df_free;
-
-	vl.values = values;
-	vl.values_len = 2;
-	sstrncpy (vl.host, hostname_g, sizeof (vl.host));
-	sstrncpy (vl.plugin, "df", sizeof (vl.plugin));
-	sstrncpy (vl.plugin_instance, "", sizeof (vl.plugin_instance));
-	sstrncpy (vl.type, type, sizeof (vl.type));
-	sstrncpy (vl.type_instance, df_name, sizeof (vl.type_instance));
-
-	plugin_dispatch_values (&vl);
-} /* void df_submit_two */
-
 __attribute__ ((nonnull(2)))
 static void df_submit_one (char *plugin_instance,
 		const char *type, const char *type_instance,
@@ -215,6 +183,9 @@ static int df_read (void)
 	{
 		unsigned long long blocksize;
 		char disk_name[256];
+		uint64_t blk_free;
+		uint64_t blk_reserved;
+		uint64_t blk_used;
 
 		if (ignorelist_match (il_device,
 					(mnt_ptr->spec_device != NULL)
@@ -274,39 +245,39 @@ static int df_read (void)
 
 		blocksize = BLOCKSIZE(statbuf);
 
-		if (report_reserved)
-		{
-			uint64_t blk_free;
-			uint64_t blk_reserved;
-			uint64_t blk_used;
+		/*
+		 * Sanity-check for the values in the struct
+		 */
+		/* Check for negative "available" byes. For example UFS can
+		 * report negative free space for user. Notice. blk_reserved
+		 * will start to diminish after this. */
+#if HAVE_STATVFS
+		/* Cast and temporary variable are needed to avoid
+		 * compiler warnings.
+		 * ((struct statvfs).f_bavail is unsigned (POSIX)) */
+		int64_t signed_bavail = (int64_t) statbuf.f_bavail;
+		if (signed_bavail < 0)
+			statbuf.f_bavail = 0;
+#elif HAVE_STATFS
+		if (statbuf.f_bavail < 0)
+			statbuf.f_bavail = 0;
+#endif
+		/* Make sure that f_blocks >= f_bfree >= f_bavail */
+		if (statbuf.f_bfree < statbuf.f_bavail)
+			statbuf.f_bfree = statbuf.f_bavail;
+		if (statbuf.f_blocks < statbuf.f_bfree)
+			statbuf.f_blocks = statbuf.f_bfree;
 
-			/* Sanity-check for the values in the struct */
-			if (statbuf.f_bfree < statbuf.f_bavail)
-				statbuf.f_bfree = statbuf.f_bavail;
-			if (statbuf.f_blocks < statbuf.f_bfree)
-				statbuf.f_blocks = statbuf.f_bfree;
+		blk_free     = (uint64_t) statbuf.f_bavail;
+		blk_reserved = (uint64_t) (statbuf.f_bfree - statbuf.f_bavail);
+		blk_used     = (uint64_t) (statbuf.f_blocks - statbuf.f_bfree);
 
-			blk_free = (uint64_t) statbuf.f_bavail;
-			blk_reserved = (uint64_t) (statbuf.f_bfree - statbuf.f_bavail);
-			blk_used = (uint64_t) (statbuf.f_blocks - statbuf.f_bfree);
-			
-			df_submit_one (disk_name, "df_complex", "free",
-					(gauge_t) (blk_free * blocksize));
-			df_submit_one (disk_name, "df_complex", "reserved",
-					(gauge_t) (blk_reserved * blocksize));
-			df_submit_one (disk_name, "df_complex", "used",
-					(gauge_t) (blk_used * blocksize));
-		}
-		else /* compatibility code */
-		{
-			gauge_t df_free;
-			gauge_t df_used;
-
-			df_free = statbuf.f_bfree * blocksize;
-			df_used = (statbuf.f_blocks - statbuf.f_bfree) * blocksize;
-
-			df_submit_two (disk_name, "df", df_used, df_free);
-		}
+		df_submit_one (disk_name, "df_complex", "free",
+				(gauge_t) (blk_free * blocksize));
+		df_submit_one (disk_name, "df_complex", "reserved",
+				(gauge_t) (blk_reserved * blocksize));
+		df_submit_one (disk_name, "df_complex", "used",
+				(gauge_t) (blk_used * blocksize));
 
 		/* inode handling */
 		if (report_inodes)
