@@ -217,7 +217,7 @@ static int do_interactive = 0;
 
 static PyThreadState *state;
 
-static PyObject *cpy_format_exception;
+static PyObject *sys_path, *cpy_format_exception;
 
 static cpy_callback_t *cpy_config_callbacks;
 static cpy_callback_t *cpy_init_callbacks;
@@ -435,8 +435,8 @@ static int cpy_write_callback(const data_set_t *ds, const value_list_t *value_li
 		sstrncpy(v->data.type_instance, value_list->type_instance, sizeof(v->data.type_instance));
 		sstrncpy(v->data.plugin, value_list->plugin, sizeof(v->data.plugin));
 		sstrncpy(v->data.plugin_instance, value_list->plugin_instance, sizeof(v->data.plugin_instance));
-		v->data.time = value_list->time;
-		v->interval = value_list->interval;
+		v->data.time = CDTIME_T_TO_DOUBLE(value_list->time);
+		v->interval = CDTIME_T_TO_DOUBLE(value_list->interval);
 		Py_CLEAR(v->values);
 		v->values = list;
 		Py_CLEAR(v->meta);
@@ -465,7 +465,7 @@ static int cpy_notification_callback(const notification_t *notification, user_da
 		sstrncpy(n->data.type_instance, notification->type_instance, sizeof(n->data.type_instance));
 		sstrncpy(n->data.plugin, notification->plugin, sizeof(n->data.plugin));
 		sstrncpy(n->data.plugin_instance, notification->plugin_instance, sizeof(n->data.plugin_instance));
-		n->data.time = notification->time;
+		n->data.time = CDTIME_T_TO_DOUBLE(notification->time);
 		sstrncpy(n->message, notification->message, sizeof(n->message));
 		n->severity = notification->severity;
 		ret = PyObject_CallFunctionObjArgs(c->callback, n, c->data, (void *) 0); /* New reference. */
@@ -976,19 +976,10 @@ PyMODINIT_FUNC PyInit_collectd(void) {
 }
 #endif
 
-static int cpy_config(oconfig_item_t *ci) {
-	int i;
+static int cpy_init_python() {
 	char *argv = "";
-	PyObject *sys, *tb;
-	PyObject *sys_path;
+	PyObject *sys;
 	PyObject *module;
-	
-	/* Ok in theory we shouldn't do initialization at this point
-	 * but we have to. In order to give python scripts a chance
-	 * to register a config callback we need to be able to execute
-	 * python code during the config callback so we have to start
-	 * the interpreter here. */
-	/* Do *not* use the python "thread" module at this point! */
 
 #ifdef IS_PY3K
 	/* Add a builtin module, before Py_Initialize */
@@ -1039,6 +1030,22 @@ static int cpy_config(oconfig_item_t *ci) {
 	PyModule_AddIntConstant(module, "NOTIF_FAILURE", NOTIF_FAILURE);
 	PyModule_AddIntConstant(module, "NOTIF_WARNING", NOTIF_WARNING);
 	PyModule_AddIntConstant(module, "NOTIF_OKAY", NOTIF_OKAY);
+	return 0;
+}
+
+static int cpy_config(oconfig_item_t *ci) {
+	int i;
+	PyObject *tb;
+
+	/* Ok in theory we shouldn't do initialization at this point
+	 * but we have to. In order to give python scripts a chance
+	 * to register a config callback we need to be able to execute
+	 * python code during the config callback so we have to start
+	 * the interpreter here. */
+	/* Do *not* use the python "thread" module at this point! */
+
+	if (!Py_IsInitialized() && cpy_init_python()) return 1;
+
 	for (i = 0; i < ci->children_num; ++i) {
 		oconfig_item_t *item = ci->children + i;
 		
@@ -1138,7 +1145,6 @@ static int cpy_config(oconfig_item_t *ci) {
 			WARNING("python plugin: Ignoring unknown config key \"%s\".", item->key);
 		}
 	}
-	Py_DECREF(sys_path);
 	return 0;
 }
 
