@@ -41,7 +41,7 @@ typedef int (*udb_query_create_callback_t) (udb_query_t *q,
  */
 int udb_query_create (udb_query_t ***ret_query_list,
     size_t *ret_query_list_len, oconfig_item_t *ci,
-    udb_query_create_callback_t cb, int legacy_mode);
+    udb_query_create_callback_t cb);
 void udb_query_free (udb_query_t **query_list, size_t query_list_len);
 
 int udb_query_pick_from_list_by_name (const char *name,
@@ -68,7 +68,7 @@ int udb_query_check_version (udb_query_t *q, unsigned int version);
 int udb_query_prepare_result (const udb_query_t const *q,
     udb_query_preparation_area_t *prep_area,
     const char *host, const char *plugin, const char *db_name,
-    char **column_names, size_t column_num, int interval);
+    char **column_names, size_t column_num, cdtime_t interval);
 int udb_query_handle_result (const udb_query_t const *q,
     udb_query_preparation_area_t *prep_area, char **column_values);
 void udb_query_finish_result (const udb_query_t const *q,
